# Obtain array ID from system
arrayid <- Sys.getenv('SLURM_ARRAY_TASK_ID')
# Different seed per array
set.seed(arrayid)

# Load parallel package
library(parallel)
# Make cluster
clus <- makeCluster(24)
# Load parallel package into cluster
clusterEvalQ(clus, library(parallel))

# Source algorithm
source("gillespie.R")
# Source intensity function (linear, type 1)
source("comp1.R")
# Source demographic parameters
source("param/param6.R")
# Source duration
source("param/times.R")

# Re-scale parameters for large size
param6$a1 <- param6$a1 / 10
param6$a2 <- param6$a2 / 10
param6$dd <- param6$dd / 10

# Set other parameters
## Poisson processes
pproc <- matrix(c(
    1, 0, 0,
    -1, 0, 0,
    -1, 1, 0,
    -1, 0, 1,
    0, -1, 0,
    0, 0, -1), ncol = 3, nrow = 6, byrow = T)
## Initial population sizes
init <- c(10000, 50, 50)
## Number of simulations
sims <- 100
## Create empty matrix
all <- matrix(NA, nrow = length(times) * 3, ncol = sims)

# Export everything to cluster
clusterExport(clus, c("gillespie",
                      "comp1",
                      "times",
                      "pproc",
                      "param6",
                      "init",
                      "sims",
                      "all"))
# Run everything
all <- parSapplyLB(clus, 1:sims, FUN = function(i){
    temp <- gillespie(init, times, param6, comp1, pproc)
    c(temp[, 2], temp[, 3], temp[, 4])})

# Split into chunks
res1 <- all[1:length(times), ]
con1 <- all[(length(times) + 1):(length(times) * 2), ]
con2 <- all[((length(times) * 2) + 1):(length(times) * 3), ]

# Save results
write(res1, file = paste("Data/unstacked/res1-large-param6", "-", arrayid, ".csv", sep = ""))
write(con1, file = paste("Data/unstacked/con1-large-param6", "-", arrayid, ".csv", sep = ""))
write(con2, file = paste("Data/unstacked/con2-large-param6", "-", arrayid, ".csv", sep = ""))
