# Load parallel package
library(parallel)
# Make cluster
clus <- makeCluster(12)
# Load parallel package into cluster
clusterEvalQ(clus, library(parallel))

# Source algorithm
source("gillespie.R")
# Source intensity function (linear, type 1)
source("comp1.R")
# Source demographic parameters
source("param/param1.R")
# Source duration
source("param/times.R")

# Set other parameters
## Poisson processes
pproc <- matrix(c(
    1, 0, 0,
    -1, 0, 0,
    -1, 1, 0,
    -1, 0, 1,
    0, -1, 0,
    0, 0, -1), ncol = 3, nrow = 6, byrow = T)
## Initial population sizes
init <- c(1000, 5, 0)
## Number of simulations
sims <- 10000
## Create empty matrix
all <- matrix(NA, nrow = length(times) * 3, ncol = sims)

# Export everything to cluster
clusterExport(clus, c("gillespie",
                      "comp1",
                      "times",
                      "pproc",
                      "param1",
                      "init",
                      "sims",
                      "all"))
# Run everything
all <- parSapplyLB(clus, 1:sims, FUN = function(i){
    temp <- gillespie(init, times, param1, comp1, pproc)
    c(temp[, 2], temp[, 3], temp[, 4])})

# Split into chunks
res1 <- all[1:length(times), ]
con1 <- all[(length(times) + 1):(length(times) * 2), ]
con2 <- all[((length(times) * 2) + 1):(length(times) * 3), ]

# Save results
write(res1, file = "Data/res1-small-param01.csv")
write(con1, file = "Data/con1-small-param01.csv")
write(con2, file = "Data/con2-small-param01.csv")
